#!/bin/sh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###########################################################BeginFile############
#
# Name:
#   chksshconn.sh
#
# Purpose:
#   Check SSH connection to remote host
#
# History:
#   081021  ScienceSoft-SA  EE0609007:  Created.
#########################################################################

#######################################################
# Usage: chksshconn.sh -h <host_name> -p <SSH_server_port> -u <user_name> -k <private_key_file_name>
#######################################################

#######################################################
# display usage information
#######################################################
if [ $# -lt 1 ]
then
    echo "Usage: chksshconn.sh -h <host_name> -p <SSH_server_port> -u <user_name> -k <private_key_file_name>"
    exit 127
fi

#######################################################
# get command line parameters
#######################################################
for i in "${@}"
do
    case $i in
        -h) OPT_HOSTNAME="$2"; shift 2;;
        -p) OPT_PORT="$2"; shift 2;;
        -u) OPT_USERNAME="$2"; shift 2;;
        -k) OPT_PRIVATEKEY="$2"; shift 2;;
    esac
done

#######################################################
# check command line parameters
#######################################################

if [ -z "$OPT_HOSTNAME" ]
then
    echo "Error: hostname parameter required. Use -h hostname."
    exit 1
fi

if [ -z "$OPT_PORT" ]
then
    echo "Error: SSH_server_port parameter required. Use -p SSH_server_port."
    exit 1
fi

if [ -z "$OPT_USERNAME" ]
then
    echo "Error: user_name parameter required. Use -u user_name."
    exit 1
fi

if [ -z "$OPT_PRIVATEKEY" ]
then
    echo "Error: private_key_file_name parameter required. Use -k private_key_file_name."
    exit 1
else
    PRIVATEKEYFILE="../run/SSHKeys/$OPT_PRIVATEKEY"
    if [ ! -f "$PRIVATEKEYFILE" ]
    then
        echo "Error: private key file \"$PRIVATEKEYFILE\" not found. Provide the correct name of the file."
	exit 1
    fi
fi

#######################################################
# check if required utilities are installed on local
# system and are in the path.
#######################################################

which tar >/dev/null 2>&1
if [ $? -ne 0 ]
then
    echo "Error: tar not found. Please, install or configure tar utility."
    exit 1
fi

which gzip >/dev/null 2>&1
if [ $? -ne 0 ]
then
    echo "Error: gzip not found. Please, install or configure gzip utility."
    exit 1
fi

which ssh >/dev/null 2>&1
if [ $? -ne 0 ]
then
    echo "Error: ssh not found. Please, install or configure ssh package."
    exit 1
fi

#######################################################
# create temp file name
#######################################################

TMPFILE=/tmp/chksshcon$$

#######################################################
# run SSH to establish connection to remote host with given parameters
# and run on the remote host "echo OK" command
#######################################################

ssh -2 -p "$OPT_PORT" -i "$PRIVATEKEYFILE" "$OPT_USERNAME@$OPT_HOSTNAME" echo OK > $TMPFILE
if [ $? -ne 0 ]
then
    echo "Error: ssh has exited with error."
    if [ -f $TMPFILE ]
    then
	rm -f $TMPFILE
    fi
    exit 1
fi

#######################################################
# check if there is a temp file with $TEMPFILE name is created
#######################################################

if [ ! -f $TMPFILE ]
then
    echo "Error: ssh was unable to run a command on the remote host."
    exit 1
fi

#######################################################
# read the content of the temp file
#######################################################

read RESPONSE < "${TMPFILE}"

#######################################################
# remove temp file
#######################################################

rm -f $TMPFILE

#######################################################
# check content of the temp file. It should be "OK"
# if not SSH failed.
#######################################################

if [ $RESPONSE != "OK" ]
then
    echo "Error: ssh failed."
    exit 1
fi

#######################################################
# SSH is OK
#######################################################

echo "OK"

exit 0
