###########################################################BeginFile############
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Time-stamp: <010411 henk-jan 11:38 genact-Cisco>
#
# Name:
#   genact-Cisco
#
# Purpose:
#   To collect log entries (from Cisco log file $OPTION2) that were added since
#   the last run and write them to file $1
#   
# Notes:
#   This script should be called from genact-main, since it depends on it to
#   set things up.
#   
# History:
#   000705 0.0.1   HJvT   EE003006:  Created (copied from genact-sulog, sulog
#                                    renamed to Cisco Log)
#   000914 0.0.10  HJvT   WE009002:  Handling of saved state improved
###########################################################EndFile##############

CEA_GENACT_CISCOLOG_STATEFILE=./genact-Cisco.ini

# Source our function library. It contains generic functions.
. $CEA_GENACT_FUNCLIB || exit 1

# We expect the name of the outfile in $1 (not checked; we trust genact-main).
OUTFILE=$1

# Define cleanup function (which is called by 'die'). This one just removes
# $OUTFILE when we're exiting unsuccessfully.
cleanup ()
{
    [[ $1 -gt 0 ]] && rm -f $OUTFILE;
}

# We expect the name of the Cisco Log file in $OPTION2.
[[ -n "$OPTION2" ]] || die 2 "Missing log platform property 'Option 2'; it should contain the Cisco Log path.";
CISCOLOG=$OPTION2

# Does the Cisco Log file exist?
if [[ ! -f $CISCOLOG ]]
then
    error "$CISCOLOG not found. An empty log will be collected."
    touch $OUTFILE
    die 0
fi

# Do we have a saved state file?
if [[ -s $CEA_GENACT_CISCOLOG_STATEFILE ]]
then
    debug "Found saved state file. Collecting new log messages."

    # Source state file and check for correctness.
    testrights $CEA_GENACT_CISCOLOG_STATEFILE "RW"
    . $CEA_GENACT_CISCOLOG_STATEFILE || die 3 "Error reading state file $CEA_GENACT_CISCOLOG_STATEFILE."

    [[ -n "$CEA_GENACT_CISCOLOG_LASTLINENUM" &&
       $CEA_GENACT_CISCOLOG_LASTLINENUM -ge 0 &&
       ( ( $CEA_GENACT_CISCOLOG_LASTLINENUM -eq 0 && -z "$CEA_GENACT_CISCOLOG_LAST3LINES" ) ||
         ( $CEA_GENACT_CISCOLOG_LASTLINENUM -gt 0 && -n "$CEA_GENACT_CISCOLOG_LAST3LINES" ) ) ]] ||
        die 4 "Invalid state file $CEA_GENACT_CISCOLOG_STATEFILE."
else
    warning "Saved state file $CEA_GENACT_CISCOLOG_STATEFILE not found or empty. This is normal the first time $CISCOLOG is collected only."

    # Initialize state variables.
    debug "Saved state file not found or empty. Initializing state."
    CEA_GENACT_CISCOLOG_LASTLINENUM=0
    CEA_GENACT_CISCOLOG_LAST3LINES=''
fi

debug "Initial state:"
debug "    CEA_GENACT_CISCOLOG_LASTLINENUM = '$CEA_GENACT_CISCOLOG_LASTLINENUM'"
debug "    CEA_GENACT_CISCOLOG_LAST3LINES  = '$CEA_GENACT_CISCOLOG_LAST3LINES'"

# Helper function: Echo line with number $1 and the 2 lines before that
# from stdin to stdout.
last3lines()
{
    LASTLINENUM=$1
    echolines $((LASTLINENUM - 2)) $LASTLINENUM
}

# In case of a race on $CISCOLOG, we'll retry NUM_RETRIES times.
NUM_RETRIES=1

while true;   # Loop is terminated using 'break'.
do
    # Determine new state of CiscoLog.
    NEW_LASTLINENUM=`linecount $CISCOLOG`
    NEW_LAST3LINES=`last3lines $NEW_LASTLINENUM < $CISCOLOG`

    debug "NEW_LASTLINENUM      = '$NEW_LASTLINENUM'"
    debug "NEW_LAST3LINES       = '$NEW_LAST3LINES'"

    PREVIOUS_LAST3LINES=`last3lines $CEA_GENACT_CISCOLOG_LASTLINENUM < $CISCOLOG`

    # Translation of saved state needed?
    if [[ $NEW_LASTLINENUM -lt $CEA_GENACT_CISCOLOG_LASTLINENUM ||
          "$PREVIOUS_LAST3LINES" != "$CEA_GENACT_CISCOLOG_LAST3LINES" ]]
    then
        info "$CISCOLOG has been truncated since the last time it was collected. Finding last collected message."
        CEA_GENACT_CISCOLOG_LASTLINENUM=0

        # First we use grep -n to find all occurrences of the last collected line
        # in the current Cisco Log.
        LASTLINE=`echo "$CEA_GENACT_CISCOLOG_LAST3LINES" | tail -1`
        grep -n "$LASTLINE" $CISCOLOG |&

        # Then we loop through the result lines, and for each line check if the
        # previous 2 collected lines match too.
        SAVED_IFS=$IFS
        IFS=':'
        while read -p number line
        do
            IFS=$SAVED_IFS
            debug "number=$number line='$line'"

            THESE_LAST3LINES=`last3lines $number < $CISCOLOG`
            STORED_LAST3LINES=`echo "$CEA_GENACT_CISCOLOG_LAST3LINES" | tail -$number`

            if [[ "$THESE_LAST3LINES" = "$STORED_LAST3LINES" ]]
            then
                debug "Matched state at line $number."
                CEA_GENACT_CISCOLOG_LASTLINENUM=$number
                [[ $number -ge 3 ]] && break  # If the last 3 lines occur in more than 1 place, the first is remembered.
            fi
            IFS=':'
        done
        IFS=$SAVED_IFS

        if [[ $CEA_GENACT_CISCOLOG_LASTLINENUM -eq 0 ]]
        then
            warning "Could not locate last collected message in $CISCOLOG. Collecting entire log."
            CEA_GENACT_CISCOLOG_LAST3LINES=''
        fi

        debug "Translated state:"
        debug "    CEA_GENACT_CISCOLOG_LASTLINENUM = '$CEA_GENACT_CISCOLOG_LASTLINENUM'"
        debug "    CEA_GENACT_CISCOLOG_LAST3LINES  = '$CEA_GENACT_CISCOLOG_LAST3LINES'"
    fi

    COLLECT_FIRSTLINENUM=$((CEA_GENACT_CISCOLOG_LASTLINENUM + 1))
    COLLECT_LINECOUNT=$((NEW_LASTLINENUM - CEA_GENACT_CISCOLOG_LASTLINENUM))

    debug "COLLECT_FIRSTLINENUM = '$COLLECT_FIRSTLINENUM'"
    debug "COLLECT_LINECOUNT    = '$COLLECT_LINECOUNT'"

#    [[ $NUM_RETRIES -eq 1 ]] && { mv $CISCOLOG ${CISCOLOG}.gone; touch $CISCOLOG; }   # Simulate race condition.

    # Copy new events from $CISCOLOG to $OUTFILE.
    echolines $COLLECT_FIRSTLINENUM $NEW_LASTLINENUM < $CISCOLOG > $OUTFILE ||
        die 5 "Error collecting log messages from $CISCOLOG to $OUTFILE."

#    [[ $NUM_RETRIES -eq 1 ]] && { rm -f $CISCOLOG; mv ${CISCOLOG}.gone $CISCOLOG; }   # Restore situation.

    EXPECTED_LAST3LINES=`echo "$NEW_LAST3LINES" | tail -$COLLECT_LINECOUNT`
    OUTFILE_LAST3LINES=`last3lines $COLLECT_LINECOUNT < $OUTFILE`

    debug "EXPECTED_LAST3LINES = $EXPECTED_LAST3LINES"
    debug "OUTFILE_LAST3LINES  = $OUTFILE_LAST3LINES"

    # Verify if we collected what we expected to collect.
    if [[ `linecount $OUTFILE` -ne $COLLECT_LINECOUNT ||  "$EXPECTED_LAST3LINES" != "$OUTFILE_LAST3LINES" ]]
    then
        debug "Contents of $OUTFILE not as expected (race condition on $CISCOLOG). Trying $NUM_RETRIES more time(s)."
        if [[ $NUM_RETRIES -eq 0 ]]
        then
            die 6 "Too many retries collecting $CISCOLOG."
        else
            warning "Race condition detected while collecting $CISCOLOG. Trying again."
            NUM_RETRIES=$((NUM_RETRIES - 1))
            sleep 1
        fi
    else
        break
    fi
done

# Update state variables.
CEA_GENACT_CISCOLOG_LASTLINENUM=$NEW_LASTLINENUM
CEA_GENACT_CISCOLOG_LAST3LINES=$NEW_LAST3LINES

debug "New state:"
debug "    CEA_GENACT_CISCOLOG_LASTLINENUM = '$CEA_GENACT_CISCOLOG_LASTLINENUM'"
debug "    CEA_GENACT_CISCOLOG_LAST3LINES  = '$CEA_GENACT_CISCOLOG_LAST3LINES'"

# Save state file.
echo "CEA_GENACT_CISCOLOG_LASTLINENUM=$CEA_GENACT_CISCOLOG_LASTLINENUM" >| $CEA_GENACT_CISCOLOG_STATEFILE
echo "CEA_GENACT_CISCOLOG_LAST3LINES='$CEA_GENACT_CISCOLOG_LAST3LINES'" >>  $CEA_GENACT_CISCOLOG_STATEFILE

# Clean up and exit successfully.
die 0
