#!/bin/sh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################
#
# Name:
#   getasegroups.sh
#
# Purpose:
#   Actuator script for Grouping Sybase ASE through SSH ES for Unix platforms.
#
# History:
#   2006dec07 ScnSoft Pipe-A  EE0609007: Created.
#   071212  0.0.27 2944109017: Removed carriage return symbols. 
###############################################################################


###########################################################
# Define debug output.
###########################################################
debug()
{
  [ -n "$CEA_DEBUG_LOG" ] \
    && echo "$SCRIPTNAME: [D] $*" >> "$CEA_DEBUG_LOG"
}

###########################################################
# Validate input parameters.
###########################################################
ValidateInputParameters()
{
  debug "Validating input parameters."

  [ $# -ge 7 ] \
    || OurExit 1 "Insufficient number of arguments."

  if [ -n "$1" ]
  then
    OUTPUTFILE="$1"
  else
    OurExit 1 "Logfile1: 'Temporary File', name of file in which to return user information data must be specified."
  fi

  [ -n "$2" ] \
    || OurExit 1 "Option1: 'Configuration File Name' must be specified."

  if [ -n "$3" ]
  then
    SERVERNAME="$3"
  else
    OurExit 1 "Option2: 'Sybase Server Name' must be specified."
  fi

  if [ -n "$4" ]
  then
    USERNAME="$4"
  else
    OurExit 1 "Option3: 'Sybase User Name' must be specified."
  fi

  PASSWORD=$(../bin/cmtool -d "${5}")

  if [ -n "$6" ]
  then
    OCS_FOLDER="$6"
  else
    OurExit 1 "Option5: 'Sybase Open Client/Server Folder' must be specified."
  fi
}

###########################################################
# Get user information data.
###########################################################
GetGroups()
{
  # Set $SYBASE environment variable.
  SYBASE=`echo "${OCS_FOLDER}" | sed -e "s|^\(.*/\)[^ 	/]\{1,\}/*$|\1|"`
  export SYBASE

  # Check if ${OCS_FOLDER}/bin/isql is existing.
  debug "Checking if ${OCS_FOLDER}/bin/isql is existing."
  if [[ ! -s ${OCS_FOLDER}/bin/isql ]]
  then
    OurExit 2 "${OCS_FOLDER}/bin/isql not found."
  fi

  test -r ${OCS_FOLDER}/bin/isql || OurExit 2 "${OCS_FOLDER}/bin/isql is not accessible for reading\executing."
  test -x ${OCS_FOLDER}/bin/isql || OurExit 2 "${OCS_FOLDER}/bin/isql is not accessible for reading\executing."
  
  # Start TSQL query using 'isql' utility.
  debug "Retrieving the user information data."
  "${OCS_FOLDER}/bin/isql" -U"${USERNAME}" -P"${PASSWORD}" -S"${SERVERNAME}" -b -w 512 -s '
' -n << !EOF! | egrep -e "[^ ]" > "${OUTPUTFILE}"

/* Set options that affect output. */
set nocount on
set statistics io, subquerycache, time off

/*
 * Example of output:
 *
 * [Persons]
 * "Iosif Vissarionovich" "Iosif Vissarionovich" "0"
 * {
 *   match logonname "stalin"
 *   match attribute "ASE_VIAZ7"
 * }
 */

select "[Persons]"
select
       '"' + fullname + '"' + ' ' + '"' + fullname + '"' + ' ' + '"0"' + char(10) +
       '{' + char(10) +
       '  match logonname "' + name + '"' + char(10) +
       '  match attribute "' + "${SERVERNAME}" + '"' + char(10) +
       '}' + char(10)
from master..syslogins where fullname is not null

/*
 * Example of output:
 * 
 * [SourceGroup]
 * "sso_role"
 * {
 *   match logonname "stalin"
 *   match attribute "ASE_VIAZ7"
 * }
 */

select "[SourceGroup]"
select
       '"' + master..syssrvroles.name + '"' + char(10) +
       '{' + char(10) +
       '  match logonname "' + master..syslogins.name + '"' + char(10) +
       '  match attribute "' + "${SERVERNAME}" + '"' + char(10) +
       '}' + char(10)
from master..syssrvroles, master..syslogins, master..sysloginroles
where master..sysloginroles.srid = master..syssrvroles.srid and master..sysloginroles.suid = master..syslogins.suid

go
!EOF!

# Check SQL query for errors.
PERSONS=`cat "${OUTPUTFILE}" | egrep -e "^\[Persons\]$"`
SOURCEGROUP=`cat "${OUTPUTFILE}" | egrep -e "^\[SourceGroup\]$"`
if [ "${PERSONS}${SOURCEGROUP}" != "[Persons][SourceGroup]" ]
then
  cat "${OUTPUTFILE}" >> "${CEA_DEBUG_LOG}"
  OurExit 2
fi
}

###########################################################
# Exit for this program.
###########################################################
OurExit()
{

  [ -n "$2" ] && debug "$2"

  case "$1"
  in
    0) debug "The collect process was successful (return code 0)."
       FinishLog
       exit 0;;
    1) debug "No log data could be collected, because of some input parameters are not valid (return code 1)."
       FinishLog
       exit 1;;
    2) debug "No log data could be collected, because of some error while the usage of Sybase DBMS features (return code 2)."
       FinishLog
       exit 2;;
    3) debug "No log data could be collected, because of error occurred (return code 3)."
       FinishLog
       exit 3;;
  esac
}

###########################################################
# Finish actuator progress log file.
###########################################################
FinishLog()
{
  echo "CeA/ExtendIT Debug Log finished by '$SCRIPTNAME' on `hostname` (`uname`), `date`." >> "${CEA_DEBUG_LOG}"
}

###########################################################
####################### << MAIN >> ########################
###########################################################

# Define debug.
SCRIPTNAME=$0
CEA_DEBUG_LOG="../log/${7}.log"
rm -f "${CEA_DEBUG_LOG}"
echo "CeA/ExtendIT Debug Log opened by '$SCRIPTNAME' on `hostname` (`uname`), `date`." >> "${CEA_DEBUG_LOG}"

# Validate input parameters.
ValidateInputParameters "$@"

# Get user information data.
GetGroups

#Exit successfully.
OurExit 0
