#!/bin/ksh

LANG=C

print_message()
{
  echo "$(date "+[%Y-%m-%d %H:%M:%S]") ${1}" >> "${2}"
}


failure()
{
  [ -z "${1}" ] || print_message "ERROR: ${1}" "${ES_LOG}"
  print_message "Processing finished [FAILURE]." "${ES_LOG}"
  exit ${2:-1}
}


stod()
{
  case "$2" in
    'Jan')
       MONTH="01"
       ;;
    'Feb')
       MONTH="02"
       ;;
    'Mar')
       MONTH="03"
       ;;
    'Apr')
       MONTH="04"
       ;;
    'May')
       MONTH="05"
       ;;
    'Jun')
       MONTH="06"
       ;;
    'Jul')
       MONTH="07"
       ;;
    'Aug')
       MONTH="08"
       ;;
    'Sep')
       MONTH="09"
       ;;
    'Oct')
       MONTH="10"
       ;;
    'Nov')
       MONTH="11"
       ;;
    'Dec')
       MONTH="12"
       ;;
     *)
       MONTH=""
       print_message "Incorrect MONTH value: '${2}'" "${ES_LOG}"
       ;;
  esac
  if [ ${#3} -lt 2 ]; then
     DAY="0${3}"
  else
     DAY=${3}
  fi
  if [ -z "$MONTH" -o -z "$DAY" ]; then
     echo ""
  else
     echo "${5}-${MONTH}-${DAY}T${4}"
  fi
}


get_file_ts()
{
  case "${OS_TYPE}" in
    'AIX')
       stod $(istat "$1" | sed -n '/modified/ {s/^[^:]*:[ 	][ 	]*\([^ 	].*\)$/\1/p;}')
       ;;
    'Linux')
       date -d "$(stat -c "%y" "${1}")" -u "+%Y-%m-%dT%H:%M:%S"
       ;;
    *)
       echo=""
       print_message "This ES does not collect from OS '${OS_TYPE}'." "${ES_LOG}"
       ;;
  esac
}


get_logs_list()
{
  >"${LOGS_LIST}"
  N=0
  for FILE in $(ls -1tr "${LOG_DIR}" 2>>"${ES_LOG}" | sed -n '/^'${FILE_PATTERN}'$/ {/'$(echo ${LOGS_GROUP} | sed 's/\./\\\./g')'/p;}'); do
      if [ -f "${LOG_DIR}/${FILE}" ]; then
         echo "${FILE}" >> "${LOGS_LIST}"
         ((N += 1))
      fi
  done
  if [ $N -eq 0 ]; then
     print_message "ERROR: Failure getting list of required logfiles for the group '${LOGS_GROUP}'." "${ES_LOG}"
     RESULT_CODE=1
  fi
  echo "${N}"
}


get_control_attr()
{
# Get attributes (filesize, modification timestamp, filename) for the second file of the logfiles list for current group,
# sorted by descend of modification timestamp (newest - first).
# This algorithm used specially for cover two types of logs rotations:
#  - insert/shift/rename
#  - append/replace cyclic
  FA=$(ls -lgot "${LOG_DIR}" 2>>"${ES_LOG}" | \
       sed -n '/^.* '${FILE_PATTERN}'$/ {s/^[^ 	]\{10\}[ 	][ 	]*[0-9][0-9]*[ 	][ 	]*\([0-9].*'${LOGS_GROUP}'.*\)$/\1/p;}' |\
       sed -n '2 {p;q;}')
  echo "${FA}"
}


get_property()
{
 sed -n '/^'${1}'=/ {s/^'${1}'=\(.*\)$/\1/p; q;}' ${PROPS_FILE} 2>>"${ES_LOG}"
 [ $? -eq 0 ] || RESULT_CODE=1
}


LOGSET=$1
LOG_TYPE=$2
LOG_DIR=$3
FILE_PATTERN=$4
PROPS_FILE=$5
RUN_MODE=$6

if [ -z "${PROPS_FILE}" ]; then
   ES_LOG="../log/gettsiemserverlogs-error.log"
   touch "${LOGSET}"
   failure "Missing property file argument."
fi

ES_NUMBER=$(basename "${PROPS_FILE}")
ES_NUMBER=${ES_NUMBER%.*}
ES_LOG="../log/${ES_NUMBER}.log"
WORK_TMP_DIR="/tmp/ES-${ES_NUMBER}"
LOGS_LIST="${WORK_TMP_DIR}/logs.lst"
TEMP_LOG="${WORK_TMP_DIR}/temp.log"
NEW_PROPS="${WORK_TMP_DIR}/${ES_NUMBER}.props"
COLLECT_PERFORMED="${WORK_TMP_DIR}/collect_performed"
OS_TYPE=$(uname)

if [ "${RUN_MODE}" == "begin" ]; then
   >"${ES_LOG}"
else
   echo "" >> "${ES_LOG}"
fi

print_message "*****************************************" "${ES_LOG}"
print_message "  Start processing ${LOG_TYPE} log" "${ES_LOG}"
print_message "*****************************************" "${ES_LOG}"

>"${LOGSET}" 2>>"${ES_LOG}" || failure "Failure creating/clearing logset."
[ -f "${PROPS_FILE}" ] || touch "${PROPS_FILE}" 2>>"${ES_LOG}" || failure "Failure creating propsfile '${PROPS_FILE}'."

if [ -z "${LOG_DIR}" ]; then
   failure "Missing log directory path argument."
else
   [ -d "${LOG_DIR}" ] || failure "Specified in ES properties directory with logfiles does not exist: '${LOG_DIR}'"
   CUR_DIR="$(pwd)"
   cd "${LOG_DIR}" 2>>"${ES_LOG}"
   if [ $? -eq 0 ]; then
      LOG_DIR="$(pwd)"
      cd "${CUR_DIR}"
   fi
   print_message "Source log directory path is '${LOG_DIR}'." "${ES_LOG}"
fi
if [ -z "${FILE_PATTERN}" ]; then
   failure "Missing regular expression of the logfiles name argument."
else
   print_message "Regular expression of the logfiles name is '${FILE_PATTERN}'." "${ES_LOG}"
fi
chmod 0600 "${LOGSET}" >/dev/null 2>>"${ES_LOG}" || failure "Failure setting strict permissions for logset file '${LOGSET}'."
if [ "${RUN_MODE}" == "begin" ]; then
   print_message "Create temporary working directory." "${ES_LOG}"
   if [ -d "${WORK_TMP_DIR}" ]; then
      rm -rf "${WORK_TMP_DIR}" >/dev/null 2>>"${ES_LOG}" || failure "Failure removing existing temporary working directory '${WORK_TMP_DIR}'."
   fi
   mkdir "${WORK_TMP_DIR}" >/dev/null 2>>"${ES_LOG}" || failure "Failure creating temporary working directory '${WORK_TMP_DIR}'."
   chmod 0700 "${WORK_TMP_DIR}" >/dev/null 2>>"${ES_LOG}" || failure "Failure setting strict permissions for temporary working directory '${WORK_TMP_DIR}'."
else
   [ -d "${WORK_TMP_DIR}" ] || failure "Temporary working directory '${WORK_TMP_DIR}' does not exist."
fi

RESULT_CODE=0
for LOGS_GROUP in $(ls -1 ${LOG_DIR} | sed -n 's/^'${FILE_PATTERN}'$/\1/p' | uniq); do
    print_message "- Collect for group '${LOGS_GROUP}' started." "${ES_LOG}"
    print_message "Reading state information from '${PROPS_FILE}'." "${ES_LOG}"
    PREV_LTS=$(get_property "${LOGS_GROUP}_MODTIME")
    [ $RESULT_CODE -eq 0 ] || break
    READ_BYTES=$(get_property "${LOGS_GROUP}_BYTESCOUNT")
    [ $RESULT_CODE -eq 0 ] || break
    FIRST_LINE=$(get_property "${LOGS_GROUP}_FIRSTPART")
    [ $RESULT_CODE -eq 0 ] || break
    if [ -z "${PREV_LTS}" -o -z "${READ_BYTES}" -o -z "${FIRST_LINE}" ]; then
       PREV_LTS=""
       print_message "Initial collect mode use." "${ES_LOG}"
    else
       print_message "Events will be collected from '${PREV_LTS}'" "${ES_LOG}"
    fi
    RESULT_CODE=0
    LAST_LTS=$PREV_LTS
    FIRST_LOG=1
    for ATTEMPT in $(seq 1 100); do
        CONTROL_ATTR=$(get_control_attr)
        [ $RESULT_CODE -eq 0 ] || break
        LOGS_NUMBER=$(get_logs_list)
        [ $RESULT_CODE -eq 0 ] || break
        if [ $LOGS_NUMBER -eq 1 -a ! -z "${CONTROL_ATTR}" ]; then
           print_message "ERROR: Failure getting attributes of required logfiles for the group '${LOGS_GROUP}'." "${ES_LOG}"
           RESULT_CODE=1
           break
        fi
        if [ $LOGS_NUMBER -gt 1 -a -z "${CONTROL_ATTR}" ]; then
           print_message "INFO: Rollover accured during logfiles list creation. Recreate list." "${ES_LOG}"
           ROTATED=1
           continue
        fi
        if [ -z "${PREV_LTS}" -o "${FIRST_LINE}" == "noline" ]; then
           FOUND_PREV=1
        else
           FOUND_PREV=0
        fi
        ROTATED=0
        LAST_ATTR=""
        for LOG_NAME in $(cat "${LOGS_LIST}"); do
            CURR_LOG="${LOG_DIR}/${LOG_NAME}"
            CURR_LTS=$(get_file_ts "${CURR_LOG}")
            if [ -z "${CURR_LTS}" ]; then
               print_message "ERROR: Failure getting timestamp for logfile '${CURR_LOG}'." "${ES_LOG}"
               RESULT_CODE=1
               break
            fi
            if [ "${CURR_LTS}" \> "${LAST_LTS}" -o "${CURR_LTS}" == "${LAST_LTS}" ]; then
               CURR_FL=$(head -n 1 "${CURR_LOG}" 2>>"${ES_LOG}")
               if [ $? -ne 0 ]; then
                  print_message "ERROR: Failure reading first line of current logfile '${CURR_LOG}'." "${ES_LOG}"
                  RESULT_CODE=1
                  break
               fi
               CURR_FL=$(echo "${CURR_FL}" | head -c 100)
               if [ $FIRST_LOG -eq 0 -a "${CURR_LTS}" == "${LAST_LTS}" -a "${FIRST_LINE}" == "${CURR_FL}" ]; then
                  FOUND_PREV=1
                  >"${TEMP_LOG}"
               else
                  if [ $FOUND_PREV -eq 0 -a "${FIRST_LINE}" != "${CURR_FL}" ]; then
                     print_message "WARNING: Last collected logfile from previous collect was not found, some events may have been lost." "${ES_LOG}"
                     print_message "INFO: Please collect more often to avoid such situations in the future." "${ES_LOG}"
                     print_message "INFO: Collect everything available for logfiles group '${LOGS_GROUP}'." "${ES_LOG}"
                     FOUND_PREV=1
                  fi
                  print_message "Collect events from logfile '${CURR_LOG}'." "${ES_LOG}"
                  cp -f "${CURR_LOG}" "${TEMP_LOG}" >/dev/null 2>>"${ES_LOG}"
                  if [ $? -ne 0 ]; then
                     print_message "ERROR: Failure copying logfile to temporary working file '${CURR_LOG}'." "${ES_LOG}"
                     RESULT_CODE=1
                     break
                  fi
                  LAST_ATTR="${CONTROL_ATTR}"
                  if [ $LOGS_NUMBER -eq 1 ]; then
                     LOGS_NUMBER=$(get_logs_list)
                     if [ $LOGS_NUMBER -eq 1 ]; then
                        CONTROL_ATTR=""
                     else
                        CONTROL_ATTR="1"
                     fi
                  else
                     CONTROL_ATTR=$(get_control_attr)
                  fi
                  [ $RESULT_CODE -eq 0 ] || break
                  if [ "${CONTROL_ATTR}" != "${LAST_ATTR}" ]; then
                     ROTATED=1
                     break
                  fi
               fi
               if [ -s "${TEMP_LOG}" ]; then
                  if [ $FOUND_PREV -eq 0 -a "${FIRST_LINE}" == "${CURR_FL}" ]; then
                     FOUND_PREV=1
                     dd if=${TEMP_LOG} ibs=${READ_BYTES} skip=1 >> "${LOGSET}" 2>/dev/null
                  else
                     cat "${TEMP_LOG}" >> "${LOGSET}" 2>>"${ES_LOG}"
                  fi
                  if [ $? -ne 0 ]; then
                     print_message "ERROR: Failure copying temporary logfile to result logset file '${TEMP_LOG}'." "${ES_LOG}"
                     # may be purge logset file and return properties to values before collect started
                     RESULT_CODE=1
                     break
                  fi
                  PREV_LTS=$CURR_LTS
                  READ_BYTES=$(ls -l "${TEMP_LOG}" | awk '{print $5}')
                  FIRST_LINE=$CURR_FL
               fi
               LAST_LTS=$CURR_LTS
               FIRST_LOG=0
            fi
        done
        [ $RESULT_CODE -eq 0 ] || break
        if [ $ROTATED -eq 0 ]; then
           break
        else
           print_message "INFO: Rollover accured during collect process. Collecting newly created logfiles." "${ES_LOG}"
        fi
    done
    if [ $RESULT_CODE -ne 0 ]; then
       print_message "Collect failure for logfile '${CURR_LOG}'." "${ES_LOG}"
    else
       if [ $ATTEMPT -eq 100 ]; then
          RESULT_CODE=1
          print_message "ERROR: Logfiles archives are produced quicker than they can be collected. Please, change log archiving settings." "${ES_LOG}"
          print_message "INFO: Please increase the size of active logfile to avoid such situation in the future and repeat the collect." "${ES_LOG}"
       fi
    fi
    if [ $RESULT_CODE -eq 0 ]; then
       if [ -z "${LAST_LTS}" ]; then
          print_message "Calculate the properties for the next collect." "${ES_LOG}"
          PREV_LTS=$(date -u "+%Y-%m-%dT%H:%M:%S")
          [ $? -eq 0 ] || failure "Failure getting current timestamp."
          READ_BYTES="0"
          FIRST_LINE="noline"
       else
          PREV_LTS=$(get_file_ts "${TEMP_LOG}")
          if [ "${PREV_LTS}" \> "${CURR_LTS}" ]; then
             PREV_LTS=$CURR_LTS
          fi
       fi
    fi
    LOGSET_SIZE=$(ls -l "${LOGSET}" | awk '{print $5}')
    print_message "Timestamp for the next collect is '${PREV_LTS}'." "${ES_LOG}"
    print_message "The total size of new collected events is ${LOGSET_SIZE} bytes." "${ES_LOG}"
    print_message "Saving current state information." "${ES_LOG}"
    echo "${LOGS_GROUP}_MODTIME=${PREV_LTS}" >> "${NEW_PROPS}"
    echo "${LOGS_GROUP}_BYTESCOUNT=${READ_BYTES}" >> "${NEW_PROPS}"
    echo "${LOGS_GROUP}_FIRSTPART=${FIRST_LINE}" >> "${NEW_PROPS}"
    if [ $RESULT_CODE -eq 0 -o $LOGSET_SIZE -gt 0 ]; then
       touch "${COLLECT_PERFORMED}"
       print_message "- Collect for group '${LOGS_GROUP}' finished [SUCCESS]." "${ES_LOG}"
    else
       print_message "- Collect for group '${LOGS_GROUP}' finished [FAILURE]." "${ES_LOG}"
    fi
done

[ $RESULT_CODE -eq 0 -a -z "$READ_BYTES" ] && print_message "ERROR: Logfiles for '${LOG_TYPE}' log not found." "${ES_LOG}"

if [ "${RUN_MODE}" == "end" ]; then
   echo "" >> "${ES_LOG}"
   if [ -f "${COLLECT_PERFORMED}" ]; then
      print_message "Saving state information in properties file." "${ES_LOG}"
      cp -f "${NEW_PROPS}" "${PROPS_FILE}"
      if [ $? -ne 0 ]; then
         failure "Failure saving new state information."
      fi
      print_message "Remove temporary working directory." "${ES_LOG}"
      rm -rf "${WORK_TMP_DIR}" >/dev/null 2>>"${ES_LOG}" || \
             print_message "WARNING: Failure removing temporary working directory '%WORK_TMP_DIR%'." "${ES_LOG}"
      print_message "Processing finished [SUCCESS]." "${ES_LOG}"
   else
      failure
   fi
fi
exit 0
