LC_ALL=C
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#!/bin/sh
###########################################################BeginFile############
# Copyright:
#   Copyright (c) IBM Corp. 1998,2010. All Rights Reserved.
#
# File-stamp: <070613 Luc 09:49 zOS_collect_events.sh>
#
# Name:
#   zOS_collect_events.sh
#
# Purpose:
#   Generic actuator script for the z/OS platform.
#   It collects 1 sublog - the audit trail.
#
# Notes:
#   The used command interpreter (shell) needs to support
#   the [[ expression ]] construct.
#   The z/OS shell, based on the KornShell, supports this construct.
#   Part of this script is based on the HPUX genact.sh script.
#
# History:
#   050831 0.0.1 luc EE0501008: Created.
#   051109 0.0.3 luc EE0501008: Extended ES and UIS configuration
#   051111 0.0.4 luc EE0501008: Improved order of alloc statements
#   051125 0.0.5 luc EE050B016: Log script start/end to console
#   060203 0.0.8 Luc EE0501008: Improved error handling; support ACF2 databases
#   060208 0.0.8 luc EE0501008: Create chunks in UTF-8
#   060213 0.0.11 luc EE0501008: Process unloaded SMF data sets
#   060224 0.0.17 luc EE0501008: Collect from data sets with a certain prefix
#   060306 0.0.18 luc UE0603004: No initial cutoff point with POLL/WAIT
#   060308 0.0.26 luc EE050B016: Send messages to the SDSF system log
#   060310 0.0.26 luc UE0603004: Optionally collect historical data
#   060310 0.0.28 luc UE0603005: Prevent Storage Management ENQ dataset removal
#   060317 0.0.33 luc UE0603004: Do not decrease cutoff dates
#   060323 0.0.36 luc EE0501008: Fixed temporary file deletion condition
#   060324 0.0.37 luc UZ0603009: Optionally use APF-authorized CARLa
#   060327 0.0.39 luc QE0412060: Added port to system log messages
#   060331 0.0.41 luc WE0603019: Keep SMF when another ES is LIVE
#   060403 0.0.43 luc WE0603019: Put agent information in MASTER data set
#   060403 0.0.44 luc WE0603019: Replace wc -c with ls -l
#   060410 0.0.47 luc WE0603019: Refresh MASTER data set more often;
#                                skip duplicate IOCONFIGs
#   060410 0.0.49 luc WE0603019: Move PORT in front of logger messages
#   060413 0.0.53 luc WE0604016: Report "compressed" or "uncompressed"
#   060413 0.0.54 luc WE0604016: Report bytes instead of megabytes
#   060424 0.0.57 luc WE0603019: Handle empty UIS/ES property directories
#   060505 0.0.60 luc EZ0604016: Customized CARLa inclusion
#   060728 0.0.69 luc QE0606003: Skip UISes conflicting with newer UISes
#   060728 0.0.70 luc QE0606003: Improved UIS skipping
#   061110 0.0.73 luc QE0606008: Set directory permission bits
#   061117 0.0.74 luc QE06S7025: Changed C2E012I, C2E013I, and C2E111I wording
#   070418 0.0.75 luc EE0704005: Light blue wash
#   070613 0.0.77 Luc EE0704005: Use sc2rsamp with zSecure < 1.8.1
#   110317        Luc EE0608002: INTERCEPT strategy
###########################################################EndFile##############

# Terminate function which removes temporary files, optionally prints a message to $STDERR, and exits.
# This function should only be called after defining $STDERR.
terminate ()
{
    rm -rf $RUN_SUBDIR
    [[ $1 -ne 0 && "$TEMP_FILE1" != "" ]] && rm -f $TEMP_FILE1
    [[ $1 -ne 0 && "$TEMP_FILE2" != "" ]] && rm -f $TEMP_FILE2
    touch $STDERR # remains empty if there is no error
    [[ -n "$3" ]] && echo "`date`: `basename $0`: $PORT: $3" >> $STDERR
    [[ -n "$3" ]] && logger -a "C2E$2I: $PORT" "$3"
    logger -a "C2E102I: $PORT" "End of `basename $0` with rc=$1; id=$ID"
    [[ $1 -ne 0 ]] && logger -a "C2E103I: $PORT" "Agent root is `dirname $PWD`"
    exit $1
}

# The next function does not log to the $STDERR file
early_terminate ()
{
    [[ -n "$3" ]] && echo "`date`: `basename $0`: $PORT: $3"
    [[ -n "$3" ]] && logger -a "C2E$2I: $PORT" "$3"
    logger -a "C2E102I: $PORT" "End of `basename $0` with rc=$1; id=$ID"
    [[ $1 -ne 0 ]] && logger -a "C2E103I: $PORT" "Agent root is `dirname $PWD`"
    exit $1
}

store_properties ()
{
    if [[ "$LAST_SUCCESS_DATE" == "" ]]
    then
        # Read previous last successful collect date (may be empty)
        touch $PARAMETER_FILE  # make sure the next command does not fail
        LAST_SUCCESS_DATE=`awk -F= '/Last successful collect/ {print $2}' $PARAMETER_FILE`
    fi
    # Store properties and last (successful) collect dates
    echo "Collect strategy=$STRATEGY" > $PARAMETER_FILE
    echo "Store raw files=$RAW" >> $PARAMETER_FILE
    echo "Last collect=$NOW" >> $PARAMETER_FILE
    echo "Last successful collect=$LAST_SUCCESS_DATE" >> $PARAMETER_FILE
    echo "Error retention=$RETPD" >> $PARAMETER_FILE
    echo "SMF Data Set Name=$SMF" >> $PARAMETER_FILE
    echo "Collect past data (discouraged)=$COLLECT_HISTORICAL_DATA" >> $PARAMETER_FILE
}

refresh_master ()
{
    # Refresh master data set (including information just written to $PARAMETER_FILE)
    echo "[Agent]" > $MASTER_CONTENTS
    # Determine agent ID
    # This assumes ../bin/db2asc ../run/startup has a single line which contains " 1.2.3.15(int)" and which looks like ... <id>(..., e.g.
    # ((objid) 1.2.3.15(int) 1) = (((uint) 80(objval) ((objid) 12.1.136(objid) 1.2.3.14(objval) ((objval) ((uint) 1(uint) 4)))))
    echo "Id=`../bin/db2asc ../run/startup | grep ' 1.2.3.15(int)' | awk -F\  '{print $8}' | awk -F\( '{print $1}'`" >> $MASTER_CONTENTS
    # Determine server name (or IP) and port
    # This assumes ../bin/db2asc ../run/startup has a single line which contains ""A" and which looks like ...:<name or IP>:<port number>"..., e.g.
    # ((objid) 1.2.3.4(int) 3) = (((uint) 1920(objval) ((int) 1(string) "12.1.1"(string) "0"(string) "A:CRMNLW016:5050"(objval) ())))
    SERVER=`../bin/db2asc ../run/startup | awk -F: '/\"A/ {print $2 "\"" $3}' | awk -F\" '{print $1 ":" $2}'`
    echo "Server=$SERVER" >> $MASTER_CONTENTS
    CFG=""
    [[ -f ../run/configuration_file_name ]] && CFG=`cat ../run/configuration_file_name`
    echo "ConfigFile=$CFG" >> $MASTER_CONTENTS
    for ES_STATE in `ls $ES_STATES | grep -v cutoff`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[EventSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $ES_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS             # ES_STATE = <x>.<y>.<z>.props
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "SMF=`awk -F= '/SMF Data Set Name/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Collect_Past_Data=`awk -F= '/Collect past data \(discouraged\)/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
    done
    for UIS_STATE in `ls $UIS_STATES`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[UserInformationSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $UIS_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS        # UIS_STATE = <x>.<y>.<z>.props
      echo "Complex=`awk -F= '/Complex name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "CKFREEZE=`awk -F= '/CKFREEZE Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "UNLOAD=`awk -F= '/UNLOAD Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
    done
    cp $MASTER_CONTENTS "//'$MASTER'"
}

# Determine agent port number
# This assumes ../bin/db2asc ../run/startup has a single line which contains "listen" and which looks like ...:...:<port number>"..., e.g.
# ((string) "listen") = (((string) "0"(string) "P:DEST_ANY:7300"(objval) ()))
PORT=`../bin/db2asc ../run/startup | awk -F: '/\"listen\"/ {print $3}' | awk -F\" '{print $1}'`

# Check if this script is executed within the correct working directory.
[[ `basename $PWD` = "run" && -d ../bin && -d ../log ]] || early_terminate 8 104 "$0 must be executed from the run directory"

# Check number of arguments
[[ $# -ge 8 ]] || early_terminate 8 105 "$0 expects at least 8 arguments"

# Determine output and log directory prefix
STATE_PREFIX=$8
ID=`echo $STATE_PREFIX | awk -F. '{print $1 "." $2 "." $3}'` # <x>.<y>.<z>.props

# Log script start to console
logger -a "C2E101I: $PORT" "Start of `basename $0`; id=$ID"
logger -a "C2E103I: $PORT" "Agent root is `dirname $PWD`"

# Determine user information source and event source state directories
UIS_STATES="../run/UIS_properties"
ES_STATES="../run/ES_properties"

# Determine event source parameter file
PARAMETER_FILE="$ES_STATES/$STATE_PREFIX"

# Determine file name components
RUN_SUBDIR="$PWD/../run/$STATE_PREFIX.tmp"
LOG_SUBDIR="$PWD/../log/$STATE_PREFIX"
NOW=`date +%Y-%m-%d.%H:%M:%S`

# Create directories if they are not yet there.
[[ -d $LOG_SUBDIR ]] || mkdir -p $LOG_SUBDIR
[[ -d $RUN_SUBDIR ]] || mkdir -p $RUN_SUBDIR
[[ -d $UIS_STATES ]] || mkdir $UIS_STATES
[[ -d $ES_STATES ]] || mkdir $ES_STATES

# Set directory permission bits
getfacl ../log | setfacl -S - $LOG_SUBDIR
getfacl ../run | setfacl -S - $RUN_SUBDIR
getfacl ../run | setfacl -S - $UIS_STATES
getfacl ../run | setfacl -S - $ES_STATES

# Determine file names
# The $CUTOFF file does *not* end with "_cutoff" in the specification. The $CUTOFF specification in the next line may therefore have to be changed.
CUTOFF="$PWD/$ES_STATES/${STATE_PREFIX}_cutoff"
STDOUT="$LOG_SUBDIR/$NOW.sh.stdout"
STDERR="$LOG_SUBDIR/$NOW.sh.stderr"
if [ -f CKR ]
then
  CKRCARLA="CKRCARLA"
  CKRCARLX="CKRCARLx"
  CKREPORT="CKREPORT"
  SCKRCARL="sckrcarl"
  SAMPLES="ckrcarla"
  CKRDSN="ckrdsn"
  CKRPREFIX="CKR"
  TCKFREEZE="CKFREEZE"
else
  CKRCARLA="C2RCARLA"
  CKRCARLX="C2RCARLx"
  CKREPORT="CNREPORT"
  SCKRCARL="sc2rsamp"
  SAMPLES="cnrsamp"
  CKRDSN="c2rdsn"
  CKRPREFIX="CNR"
  TCKFREEZE="IOCONFIG"
fi
SYSPRINT="$LOG_SUBDIR/$NOW.$CKRCARLA.SYSPRINT"
CKREPORTF="$LOG_SUBDIR/$NOW.$CKREPORT"
CPU="$RUN_SUBDIR/CPU"                                       # tmp
MAPPER_CONFIG="$RUN_SUBDIR/mapper_config"                   # tmp
MAPPER_CONFIG_RACF="$RUN_SUBDIR/mapper_config_racf"         # tmp
MAPPER_CONFIG_ACF2="$RUN_SUBDIR/mapper_config_acf2"         # tmp
MASTER_CONTENTS="$RUN_SUBDIR/master_contents"               # tmp
MAXDATE="$RUN_SUBDIR/MAXDATE"                               # tmp
TMP_CUTOFF="$RUN_SUBDIR/tmp_cutoff"                         # tmp
ALL_CUTOFF="$RUN_SUBDIR/all_cutoff"                         # tmp

# Make sure $STDOUT exists, so information can be appended to it.
touch $STDOUT

# Read and check temporary file parameters
TEMP_FILE1=$1
TEMP_FILE2=$2 # Now employed for C2EXREF
[[ ! -z "$TEMP_FILE1" ]] || terminate 8 106 "Name of Temporary file 1 is not given."
[[ ! -z "$TEMP_FILE2" ]] || terminate 8 107 "Name of Temporary file 2 is not given."

# Read properties
STRATEGY=$3
RAW=$4
RETPD=$5
SMF=$6
COLLECT_HISTORICAL_DATA=$7

# Convert some properties to uppercase
STRATEGY=`echo $STRATEGY | tr '[:lower:]' '[:upper:]'`
RAW=`echo $RAW | tr '[:lower:]' '[:upper:]'`
SMF=`echo $SMF | tr '[:lower:]' '[:upper:]'`
COLLECT_HISTORICAL_DATA=`echo $COLLECT_HISTORICAL_DATA | tr '[:lower:]' '[:upper:]'`

# Check properties
if [[ "$STRATEGY" != "LIVE" && "$STRATEGY" != "INTERCEPT" && "$STRATEGY" != "POLL" && "$STRATEGY" != "WAIT" ]] # EE0608002
then
    terminate 8 108 "The \"Collect strategy\" parameter is $STRATEGY but should have been LIVE, INTERCEPT, POLL, or WAIT." # EE0608002
fi

[[ "$RAW" == "YES" || "$RAW" == "NO" ]] || terminate 8 109 "The \"Store raw files\" parameter is $RAW but should have been YES or NO."

[[ ! -z "$RETPD" ]] || terminate 8 110 "An error retention period should have been specified."

## For future implementation:
#if [[ "$RAW" == "YES" && -z "$TEMP_FILE3" ]]
#then
#    terminate 8 199 "The \"Store raw files\" parameter is $RAW but the Name of Temporary file 2 is not given."
#fi

if [[ "$STRATEGY" == "LIVE" || "$STRATEGY" == "INTERCEPT" ]]
then
    [[ -z "$SMF" ]] || terminate 8 111 "With \"Collect strategy\" $STRATEGY, an SMF Data Set Name specification is not allowed."
else # POLL or WAIT
    [[ ! -z "$SMF" ]] || terminate 8 112 "With \"Collect strategy\" $STRATEGY, an SMF Data Set Name must be specified."
fi

[[ "$COLLECT_HISTORICAL_DATA" == "YES" || "$COLLECT_HISTORICAL_DATA" == "NO" ]] || terminate 8 113 "The \"Collect historical data\" parameter is $COLLECT_HISTORICAL_DATA but should have been YES or NO."

# Determine MASTER and ENQ data set names, as well as some of their qualifiers
MASTER=`cat ../run/master`
[[ "$MASTER" != "" ]] || terminate 8 114 "The master file in the run directory does not exist or is empty"
C2ELVPFX=`echo $MASTER | awk -F. '{prefix = ""}{for(i=1; i<=NF && $i != "MASTER"; i++) {prefix = prefix $i "."}}{print prefix}'`
MASTERQUAL=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)} {if(i <= NF) {print "MASTER"}}'`
[[ "$MASTERQUAL" != "" ]] || terminate 8 115 "The master data set specified in the master file in the run directory should have a MASTER qualifier"
C2ELVLLQ=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)}{suffix = ""}{for(j=i+1; j<=NF; j++){suffix = suffix "." $j}}{print suffix}'`
ENQ="${C2ELVPFX}ENQ$C2ELVLLQ"

# Store properties and refresh master data set (so current data are known even if CARLa fails)
store_properties
refresh_master

# Determine whether at least one ES has strategy LIVE or INTERCEPT # EE0608002
LIVE_OR_INTERCEPT_ES_EXISTS=0 # EE0608002
for ES_STATE in `ls $ES_STATES | grep -v cutoff`
do
  [[ "`awk -F= '/Collect strategy/ {print $2}' $ES_STATES/$ES_STATE`" == "LIVE" || "`awk -F= '/Collect strategy/ {print $2}' $ES_STATES/$ES_STATE`" == "INTERCEPT" ]] && LIVE_OR_INTERCEPT_ES_EXISTS=1
done

# The following file removals should not be necessary. (Or are they necessary in case noclobber is set?)
rm -f $TEMP_FILE1
rm -f $TEMP_FILE2

[[ -f CKR || -s ../run/license ]] || terminate 8 116 "The license file in the run directory does not exist or is empty"
[[ -s ../run/$SCKRCARL ]] || terminate 8 117 "The $SCKRCARL file in the run directory does not exist or is empty"

rm -f $MAPPER_CONFIG
[[ -f CKR ]] || echo "alloc type=input dd=license dsn='`cat ../run/license`'" >> $MAPPER_CONFIG
echo "alloc type=input dd=$SAMPLES dsn='`cat ../run/$SCKRCARL`'" >> $MAPPER_CONFIG
echo "alloc type=input dd=c2ecntl path='$CUTOFF'" >> $MAPPER_CONFIG
# Embed custom sample library
[[ -s ../run/c2esamp ]] && echo "alloc type=input dd=c2esamp dsn='`cat ../run/c2esamp`'" >> $MAPPER_CONFIG
echo "alloc type=input dd=c2eixes path='$PWD/../run/c2eixes'" >> $MAPPER_CONFIG
echo "alloc type=input dd=c2ei0es path='$PWD/../run/c2ei0es'" >> $MAPPER_CONFIG
echo "alloc type=input dd=c2eices path='$PWD/../run/c2eices'" >> $MAPPER_CONFIG

if [[ "$STRATEGY" == "LIVE" || "$STRATEGY" == "INTERCEPT" ]] # EE0608002
then
    echo "option serialization(wait,maxwait(30),enq($CKRDSN,sysdsn),unit,volser)" >> $MAPPER_CONFIG
    echo "alloc type=smf dsnpref=${C2ELVPFX}smf. delete" >> $MAPPER_CONFIG
    if [[ "$STRATEGY" == "LIVE" ]] # EE0608002
    then
      echo "alloc type=smf active" >> $MAPPER_CONFIG
    fi
elif [[ "$STRATEGY" == "POLL" ]]
then
    echo "deftype type=smf$" >> $MAPPER_CONFIG
    echo "option serialization(fail,enq($CKRDSN,sysdsn))" >> $MAPPER_CONFIG
    [[ $LIVE_OR_INTERCEPT_ES_EXISTS -eq 0 ]] && echo "alloc type=smf$ dsnpref=${C2ELVPFX}smf. delete" >> $MAPPER_CONFIG # EE0608002
    echo "alloc type=smf dsn=$SMF" >> $MAPPER_CONFIG
else # "$STRATEGY" == "WAIT"
    echo "deftype type=smf$" >> $MAPPER_CONFIG
    echo "option serialization(wait,maxwait(30),enq($CKRDSN,sysdsn),unit,volser)" >> $MAPPER_CONFIG
    [[ $LIVE_OR_INTERCEPT_ES_EXISTS -eq 0 ]] && echo "alloc type=smf$ dsnpref=${C2ELVPFX}smf. delete" >> $MAPPER_CONFIG # EE0608002
    echo "alloc type=smf dsn=$SMF" >> $MAPPER_CONFIG
fi

# Obtain CKFREEZE and UNLOAD information from user information sources

UIS_LIST=`ls -t $UIS_STATES`   # ordered from new to old
UIS_NUMBER=$((`echo $UIS_LIST | wc -w`))
[[ $UIS_NUMBER -gt 0 ]] || terminate 8 119 "Collect SMF terminated; User Information Source configuration incomplete"
I=0
for UIS_STATE in $UIS_LIST     # initialize arrays with UIS information
do
  INCLUDE_UIS[$I]=1            # include each UIS by default
  INCLUDE_DATABASE[$I]=1       # include each database by default
  UIS_NAME[$I]=$UIS_STATE
  UIS_COMPLEX[$I]=`awk -F= '/Complex name/ {print $2}' $UIS_STATES/$UIS_STATE`
  UIS_CKFREEZE[$I]=`awk -F= '/CKFREEZE Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`
  UIS_STRATEGY[$I]=`awk -F= '/Collect strategy/ {print $2}' $UIS_STATES/$UIS_STATE`
  UIS_UNLOAD[$I]=`awk -F= '/UNLOAD Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`
  I=$(($I + 1))
done

I=0
while [[ $I -lt $UIS_NUMBER ]]           # traverse UISes to find out what should be skipped
do
  J=0
  while [[ $J -lt $I ]]                  # traverse more recently configured UISes
  do
    if [[ ${INCLUDE_UIS[$J]} -eq 1 ]]    # the newer UIS will be included
    then
        if [[ ("${UIS_STRATEGY[$I]}" == "LIVE" || "${UIS_STRATEGY[$I]}" == "INTERCEPT") && ("${UIS_STRATEGY[$J]}" == "LIVE" || "${UIS_STRATEGY[$J]}" == "INTERCEPT") ]] # EE0608002
        then
            INCLUDE_UIS[$I]=0            # skip the older UIS
            echo "Skipped ${UIS_NAME[$I]} because it has strategy LIVE or INTERCEPT, just like ${UIS_NAME[$J]}" # EE0608002
            logger -a "C2E121I: $PORT" "Skipped ${UIS_NAME[$I]} because it has strategy LIVE or INTERCEPT, just like ${UIS_NAME[$J]}" # EE0608002
        elif [[ "${UIS_CKFREEZE[$I]}" == "${UIS_CKFREEZE[$J]}" ]]
        then
            INCLUDE_UIS[$I]=0            # skip the older UIS
            echo "Skipped ${UIS_NAME[$I]}, sharing ${UIS_CKFREEZE[$I]} with ${UIS_NAME[$J]}"
            logger -a "C2E118I: $PORT" "Skipped ${UIS_NAME[$I]}, sharing ${UIS_CKFREEZE[$I]} with ${UIS_NAME[$J]}"
        elif [[ "${UIS_UNLOAD[$I]}" == "${UIS_UNLOAD[$J]}" ]]
        then
            if [[ "${UIS_COMPLEX[$I]}" != "${UIS_COMPLEX[$J]}" ]]
            then
                INCLUDE_UIS[$I]=0        # skip the older UIS
                echo "Skipped ${UIS_NAME[$I]}, using the same database as ${UIS_NAME[$J]} but a different COMPLEX"
                logger -a "C2E122I: $PORT"  "Skipped ${UIS_NAME[$I]}, using the same database as ${UIS_NAME[$J]} but a different COMPLEX"
            else
                INCLUDE_DATABASE[$I]=0   # do not include a database twice
            fi
        fi
    fi
    J=$(($J + 1))
  done
  I=$(($I + 1))
done

I=0
while [[ $I -lt $UIS_NUMBER ]]           # traverse UISes to allocate CKFREEZEs and databases
do
  if [[ ${INCLUDE_UIS[$I]} -eq 1 ]]
  then
      COMPLEX=${UIS_COMPLEX[$I]}
      [[ "$COMPLEX" != "" ]] && COMPLEX="complex=$COMPLEX"
      echo "alloc type=$TCKFREEZE dsn=${UIS_CKFREEZE[$I]} $COMPLEX" >> $MAPPER_CONFIG
      if [[ ${INCLUDE_DATABASE[$I]} -eq 1 ]] # not included already
      then
          if [[ "${UIS_UNLOAD[$I]}" != "" ]]
          then
              echo "alloc type=unload dsn=${UIS_UNLOAD[$I]} $COMPLEX" >> $MAPPER_CONFIG
          else # PRIMARY ACTIVE
              echo "alloc type=racf primary active $COMPLEX" > $MAPPER_CONFIG_RACF
              echo "alloc type=acf2 primary active $COMPLEX" > $MAPPER_CONFIG_ACF2
              echo "include esm=racf path='$MAPPER_CONFIG_RACF'" >> $MAPPER_CONFIG
              echo "include esm=acf2 path='$MAPPER_CONFIG_ACF2'" >> $MAPPER_CONFIG
          fi
      fi
  fi
  I=$(($I + 1))
done

UN="un"
# if the temp file name ends in .gz then the file contents must be gzipped
case $TEMP_FILE1 in
  *.gz) COMPRESS1="compress=gzip"
        UN="";;
esac
case $TEMP_FILE2 in
  *.gz) COMPRESS2="compress=gzip";;
esac

cat >> $MAPPER_CONFIG <<EOF
alloc type=output dd=c2elog path='$TEMP_FILE1'
fileoption dd=c2elog encoding=utf-8 $COMPRESS1
alloc type=output dd=$CKREPORT path='$CKREPORTF'
alloc type=output dd=c2exref path='$TEMP_FILE2'
fileoption dd=c2exref encoding=utf-8 $COMPRESS2
alloc type=output dd=maxdate path='$MAXDATE'
i m=C2ELES
deftype type=enq$
alloc type=enq$ dd=enq dsn=$ENQ
newlist type=enq$ outlim=0
sortlist record
EOF

# Determine cutoff point for input
[[ -f $CUTOFF ]] || echo "exclude datetime<today-2" > $CUTOFF
if [[ "$COLLECT_HISTORICAL_DATA" == "YES" ]]
then
    mv $CUTOFF $TMP_CUTOFF
    touch $CUTOFF
else
    touch $TMP_CUTOFF
fi

# The $CUTOFF name in the next log message is very long.
# Something like "18.1.113.props/CUTOFF" or "run/18.1.113.props/CUTOFF" would have been sufficient.
CUTOFF_NOW=`date`
echo "$CUTOFF_NOW: Collect SMF started; cutoff time stamps are read from state file $CUTOFF:" >> $STDOUT
echo "$CUTOFF_NOW: System Cutoff" >> $STDOUT
grep -v today $CUTOFF | awk -F= -v NOW="$CUTOFF_NOW" '{print NOW ": " substr(substr($2, 1, index($2, " ") - 1) "    ", 1, 4) "   " substr($3, 2, length($3) - 2)}' >> $STDOUT

# Run CARLa
../run/$CKRCARLX "i path='$MAPPER_CONFIG'" < /dev/null > $SYSPRINT 2> $STDERR
RETURN_CODE=$?

if [[ $RETURN_CODE -ge 12 ]]
then
    # Read previous last successful collect date (may be empty)
    touch $PARAMETER_FILE  # make sure the next command does not fail
    LAST_SUCCESS_DATE=`awk -F= '/Last successful collect/ {print $2}' $PARAMETER_FILE`
else
    LAST_SUCCESS_DATE=$NOW
fi

store_properties
refresh_master

# Reset cutoff point and return on error
if [[ $RETURN_CODE -ge 12 ]]
then
    [[ "$COLLECT_HISTORICAL_DATA" == "YES" ]] && mv $TMP_CUTOFF $CUTOFF
    terminate 8 120 "Collect SMF error occurred in $CKRCARLA - see messages above (and `basename $SYSPRINT`)"
fi

chmod a-x $CKREPORTF

# Determine maximum cutoff dates for all systems
# Each line is supposed to look like "exclude system=DINO datetime<=' 2 Jun 2000 04:32:05.92'" (or like "... today ...")
cat $CUTOFF $TMP_CUTOFF $MAXDATE | grep -v "today" > $ALL_CUTOFF
awk -F\' '{print $2 "?" $1}' $ALL_CUTOFF | sort -k5,5 -k3,3n -k2,2M -k1,1n -k4,4 | awk '{ line[NR] = $0 } END { for ( i = NR; i > 0; i--) print line[i] }' | uniq -f 4 | awk -F? '{ print $2 "'\''" $1 "'\''" }' > $CUTOFF
cut -d " " -f 2 $CUTOFF | cut -d "=" -f 2 | awk '{ line[NR] = $0 } END {lines = ""; if(NR > 0) {lines = line[1]}; for(i = 2; i <= NR; i++) {lines = lines "," line[i]}; print "exclude system<>(" lines ") datetime<today-2"}' > $ALL_CUTOFF
cat $ALL_CUTOFF >> $CUTOFF

# In the next line, "date" is almost duplicated as the "at" date. Also, the saved log might be mentioned as well.
# Besides, the new cutoff date could be mentioned.
CUTOFF_NOW=`date`
echo "$CUTOFF_NOW: Collect SMF finished successfully at `date +%Y-%m-%d,%H:%M:%S`, saved state file $CUTOFF:" >> $STDOUT
echo "$CUTOFF_NOW: System Cutoff" >> $STDOUT
grep -v today $CUTOFF | awk -F= -v NOW="$CUTOFF_NOW" '{print NOW ": " substr(substr($2, 1, index($2, " ") - 1) "    ", 1, 4) "   " substr($3, 2, length($3) - 2)}' >> $STDOUT

# The next lines assume the CKR0039 message looks like "CKR0039 00 <program_name> used 3.13 CPU seconds, 5,248KB, and took 9.5 wall clock seconds".
# A conversion of "5,248KB," to an integer number of megabytes is possible but the $TEMP_FILE1 size may be more interesting and is reported now.
# It is also assumed that ls -l $TEMP_FILE1 output looks like "... ... ... ... <size> ......".
grep -e "^${CKRPREFIX}0039 00" $STDERR | head -n 1 > $CPU
echo "`date`: Collect SMF script took `awk '{print $5}' $CPU` CPU seconds, `ls -l $TEMP_FILE1 | awk '{print $5}'` bytes ${UN}compressed, and `awk '{print $11}' $CPU` wall clock seconds" >> $STDOUT

# remove log files older than $RETPD days
find $LOG_SUBDIR -type f -mtime +$RETPD | xargs rm

# Clean up and exit with success.
terminate 0
